/*
 * Copyright (C) 2005 Mario Strasser <mast@gmx.net>,
 *                    Swiss Federal Institute of Technology (ETH) Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: mac.h,v 1.1 2005/08/15 10:01:44 mast2 Exp $
 */

/**
 * @file mac.h
 * @author Mario Strasser <mast@gmx.net>
 * @brief Generic Chipcon CC1000 MAC-Interface
 */

#ifndef _MAC_H_
#define _MAC_H_

#include <sys/types.h>

/** mask for the cluster (network) part of a node address */
#define CLUSTER_MASK        0xffe0
/** mask for the node part of a node address */
#define NODE_MASK           (~CLUSTER_MASK)
/** broadcast address */
#define BROADCAST_ADDR      0xffff
/** multicast address for address a */
#define MULTICAST_ADDR(a)   (((a) & CLUSTER_MASK)|(BROADCAST_ADDR & NODE_MASK))

/**
 * Structure describing the CC1000 MAC-Interface.
 */
typedef struct mac_interface_st {

    /** 
     * Returns the configured mac address. 
     *
     * @return configured mac address.
     */
    u_short (*get_address)(void);

    /** 
     * Returns the maximal payload size of a MAC packet.
     *
     * @return maximal payload size of a MAC packet. 
     */
    u_short (*get_mtu)(void);

    /**
     * Sends a MAC packet.
     *
     * @param dst [in] destination address
     * @param data [in] payload of the packet
     * @param data_len [in] payload size
     * @return 0 on success, -1 otherwise.
     */
    int (*send)(u_short dst, u_char *data, u_short data_len);

    /**
     * Receives a MAC packet.
     *
     * @param src [out] source address
     * @param dst [out] destination address
     * @param data [out] payload of the packet
     * @param data_length [in] maximal size of the payload
     *                    [out] actual size of the payload
     * @param ms [in] maximal duration (in ms) to wait for a packet
     * @return 0 on success, -1 on timeout.
     */
    int (*receive)(u_short *src, u_short *dst, u_char *data,
                   u_short *data_length, u_long ms);

} mac_interface_t;

#endif /* _MAC_H_ */
