/*
 * Copyright (C) 2000-2004 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch
 *
 */
 /*
 * btn-suart.h
 *
 * 18.06.2004 Martin Hinz <btnode@hinz.ch>
 */
#ifndef _BTN_SUART_H_
#define _BTN_SUART_H_

/**

\file btn-suart.h

\author Martin Hinz <btnode@hinz.ch>

\date 18.06.2004

\brief Software UART device driver.

This driver uses an internal counter to produce and read the UART bit stream
connected to the pin defined in the btn-hardware.h file. Only low bauds are
working reliable (2400baud till 9600baud) because for every bit an interrupt
handle has to be called. Interrupts can not be executed if the system is
already in a interrupt handle but timing is very important for this driver.\n
Read and write at the same time is not possible because both use the same
internal counter.\n
\attention The NutOS sets threads to sleep if the output buffer is full!

See hardware header file \link arch/btn-hardware.h btn-hardware.h\endlink for connection.

Use this driver like the hardware uart of the nutos:
\verbatim
FILE *uarts;
long sbaud = 2400;
NutRegisterDevice(&S_UART, 0, 0);
uarts = fopen(S_UART.dev_name, "r+");
_ioctl(_fileno(uarts), UART_SETSPEED, &sbaud);
\endverbatim
(default baud is set to 9600)

*/

#include <sys/device.h>
#include <dev/uart.h>
#include <dev/usart.h>



#ifdef __BTN_UNIX__

/** on unix emulation we map S_UART to devUsartAvr2 */
extern NUTDEVICE devUsartAvr2;
#define S_UART devUsartAvr2

#else

extern NUTDEVICE devUsartAvrS;
#define S_UART devUsartAvrS

#endif

#endif
