/*
 * Copyright (C) 2000-2005 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch
 *
 */

/**
* \file rprog.h
*
* \date 29.04.2005
*
* \author Mustafa Yuecel <yuecelm@ee.ethz.ch>
*
* @brief add ability for remote programming (over L2CAP)
*
*/

#ifndef _RPROG_H_
#define _RPROG_H_

#include <sys/types.h>
#include <bt/bt_l2cap.h>

// flags
#define ACK                   0
#define DATA                  1   // 1 byte flag, 1 byte sequence number, rest payload
#define END                   2
#define NACK                  128

// commands
#define PUT_PROG_MODE         2
#define PUT_PROG_MODE_LEN     4   // 1 byte command, 3 bytes length
#define PUT_PROG_MODE_ARG     3
#define PUT_PROG_MODE_ARG_LEN 5   // 1 byte command, 3 bytes length, 1 byte arg
#define CANCEL_PROG_MODE      4
#define CANCEL_PROG_MODE_LEN  1
#define GET_MAX_MTU           8
#define GET_MAX_MTU_LEN       1
#define DATA_LEN              3
#define ACK_LEN               5
#define CRC_LEN               2
// modes
#define PROG_MODE_READY       0
#define PROG_MODE_PROGRAMMING 1 
#define PROG_MODE_PROGRAMMED  2


#define RP_RESET_CB u_char (*reset_cb) (u_char arg)

/**
* this function inits an L2CAP channel ready for remote programming.
* look at app/rprog/demo-rprog.c for an example.
*
* @param l2cap pointer to l2cap_stack (initialized)
* @param reset_cb callback for reset request
*
* @return 0 if no error
*/
u_char bt_remoteprog_init(struct bt_l2cap_stack* l2cap_stack, RP_RESET_CB);

#endif
